#! /usr/bin/env python
# -*- coding: utf-8 -*-
import time
import pathlib
import zipfile
import geopandas as gpd
from urllib.request import urlretrieve

# Logging
import logging
logger = logging.getLogger(__name__)


class BaseDownloader(object):
    THIS_DIR = pathlib.Path(__file__).parent
    PARENT_DIR = THIS_DIR.parent

    def __init__(self, data_dir=None):
        # Set the download directory
        if data_dir:
            self.data_dir = pathlib.Path(str(data_dir))
        else:
            self.data_dir = self.PARENT_DIR.joinpath("data")

        # Initialize all the directories we will need
        if not self.data_dir.exists():
            self.data_dir.mkdir()
        self.raw_dir = self.data_dir.joinpath("raw")
        if not self.raw_dir.exists():
            self.raw_dir.mkdir()
        self.processed_dir = self.data_dir.joinpath("processed")
        if not self.processed_dir.exists():
            self.processed_dir.mkdir()

        # Set file paths
        self.set_paths()

    def set_paths(self):
        self.shp_name = self.zip_name.replace(".zip", ".shp")
        self.shp_path = self.raw_dir.joinpath(self.shp_name)
        self.zip_path = self.raw_dir.joinpath(self.zip_name)
        self.geojson_name = f"{self.PROCESSED_NAME}.geojson"
        self.geojson_path = self.processed_dir.joinpath(self.geojson_name)

    def run(self):
        self.download()
        self.unzip()
        self.process()

    def download(self):
        """
        Downloads the TIGER SHP file of Census block for the provided state and county.
        Returns the path to the ZIP file.
        """
        # Check if the zip file already exists
        if self.zip_path.exists():
            logger.debug(f"ZIP file already exists at {self.zip_path}")
            return

        # If it doesn't, download it from the Census FTP
        logger.debug(f"Downloading {self.url} to {self.zip_path}")
        urlretrieve(self.url, self.zip_path)
        time.sleep(1)

    def unzip(self):
        """
        Unzip the provided ZIP file.
        """
        # Check if the shape has already been unzipped
        if self.shp_path.exists():
            logger.debug(f"SHP already unzipped at {self.shp_path}")
            return

        # If not, unzip it now.
        logger.debug(f"Unzipping {self.zip_path} to {self.raw_dir}")
        with zipfile.ZipFile(self.zip_path, "r") as z:
            z.extractall(self.raw_dir)

    def process(self):
        """
        Refine the raw data and convert it to our preferred format, GeoJSON.
        """
        # Check if the geojson file already exists
        if self.geojson_path.exists():
            logger.debug(f"GeoJSON file already exists at {self.geojson_path}")
            return

        # Write out GeoJSON file
        gdf = gpd.read_file(self.shp_path)
        logger.debug(f"Writing out {len(gdf)} shapes to {self.geojson_path}")
        gdf.to_file(self.geojson_path, driver="GeoJSON")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import us
import geopandas as gpd
from census_map_downloader.base import BaseDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class StatePlacesDownloader2018(BaseDownloader):
    """
    Download 2018 places for a single state.
    """
    YEAR = 2018

    def __init__(self, state, data_dir):
        # Configure the state
        self.state = us.states.lookup(state)
        super().__init__(data_dir)

    def set_paths(self):
        self.shp_name = self.zip_name.replace(".zip", ".shp")
        self.shp_path = self.raw_dir.joinpath(self.shp_name)
        self.zip_path = self.raw_dir.joinpath(self.zip_name)

    def run(self):
        self.download()
        self.unzip()
        return self.shp_path

    @property
    def url(self):
        return f"https://www2.census.gov/geo/tiger/TIGER{self.YEAR}/PLACE/tl_{self.YEAR}_{self.state.fips}_place.zip"

    @property
    def zip_name(self):
        return f"tl_{self.YEAR}_{self.state.fips}_place.zip"


class PlacesDownloader2018(BaseDownloader):
    """
    Download all 2010 tracts in the United States.
    """
    YEAR = 2018
    PROCESSED_NAME = "places_2018"

    def __init__(self, data_dir=None):
        super().__init__(data_dir=data_dir)
        self.merged_path = self.raw_dir.joinpath("places_us_2018.shp")

    def run(self):
        self.download()
        self.process()

    def set_paths(self):
        self.geojson_name = f"{self.PROCESSED_NAME}.geojson"
        self.geojson_path = self.processed_dir.joinpath(self.geojson_name)

    def download(self):
        if self.merged_path.exists():
            logger.debug(f"SHP file already exists at {self.merged_path}")
            return

        # Loop through all the states and download the shapes
        path_list = []
        for state in us.STATES:
            logger.debug(f"Downloading {state}")
            shp_path = StatePlacesDownloader2018(
                state.abbr,
                data_dir=self.data_dir
            ).run()
            path_list.append(shp_path)

        # Open all the shapes
        df_list = [gpd.read_file(p) for p in path_list]

        # Concatenate them together
        df = gpd.pd.concat(df_list)

        logger.debug(f"Writing file with {len(df)} tracts to {self.merged_path}")
        df.to_file(self.merged_path, index=False)

    def process(self):
        """
        Refine the raw data and convert it to our preferred format, GeoJSON.
        """
        if self.geojson_path.exists():
            logger.debug(f"GeoJSON file already exists at {self.geojson_path}")
            return

        gdf = gpd.read_file(self.merged_path)
        logger.debug(f"Writing out {len(gdf)} shapes to {self.geojson_path}")
        gdf.to_file(self.geojson_path, driver="GeoJSON")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import us
import geopandas as gpd
from census_map_downloader.base import BaseDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class StateTractsDownloader2010(BaseDownloader):
    """
    Download 2010 tracts for a single state.
    """
    def __init__(self, state, data_dir):
        # Configure the state
        self.state = us.states.lookup(state)
        super().__init__(data_dir)

    @property
    def url(self):
        return self.state.shapefile_urls("tract")

    @property
    def zip_name(self):
        return f"tl_2010_{self.state.fips}_tract10.zip"


class StateTractsDownloader2000(StateTractsDownloader2010):
    """
    Download 2000 tracts for a single state.
    """
    @property
    def url(self):
        return f"https://www2.census.gov/geo/tiger/TIGER2009/{self.zip_folder}/{self.zip_name}"

    @property
    def zip_name(self):
        return f"tl_2009_{self.state.fips}_tract00.zip"

    @property
    def zip_folder(self):
        return f"{self.state.fips}_{self.state.name.upper().replace(' ', '_')}"


class TractsDownloader2010(BaseDownloader):
    """
    Download all 2010 tracts in the United States.
    """
    PROCESSED_NAME = "tracts_2010"

    def run(self):
        self.download()
        self.process()

    def download(self):
        # Loop through all the states and download the shapes
        path_list = []
        for state in us.STATES:
            logger.debug(f"Downloading {state}")
            shp_path = StateTractsDownloader2010(
                state.abbr,
                data_dir=self.data_dir
            ).run()
            path_list.append(shp_path)

        # Open all the shapes
        df_list = [gpd.read_file(p) for p in path_list]

        # Concatenate them together
        df = gpd.pd.concat(df_list)

        # Write it out, if it doesn't already exist.
        us_path = self.data_dir.joinpath("us.shp")
        if us_path.exists():
            logger.debug(f"File already exists at {us_path}")
            return us_path
        logger.debug(f"Writing file with {len(df)} tracts to {us_path}")
        df.to_file(us_path, index=False)
        return us_path



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from census_map_downloader.base import BaseDownloader

# Logging
import logging
logger = logging.getLogger(__name__)


class CountiesDownloader2018(BaseDownloader):
    """
    Download counties.
    """
    PROCESSED_NAME = "counties_2018"

    @property
    def url(self):
        return "https://www2.census.gov/geo/tiger/TIGER2018/COUNTY/tl_2018_us_county.zip"

    @property
    def zip_name(self):
        return f"tl_2018_us_county.zip"
